-----------------------------------------------------------------------------------------------
-- Client Lua Script for CurrencyMonitor
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

-- CRB_CurrencySprites:sprMediumCashPlatinum
-- CRB_CurrencySprites:sprArtTier3


require "Window"
 
-----------------------------------------------------------------------------------------------
-- CurrencyMonitor Module Definition
-----------------------------------------------------------------------------------------------
local CurrencyMonitor = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local bNewSession = true
local bSessionReset = false
local iSessionCash = 0
local bHasBothFactions = false
local bHasExile = false
local bHasDominion = false

local bIsMaximized = true

local strCurrencySel = "default"

local kclrWhite = ApolloColor.new("white")
local kclrRed 	= ApolloColor.new("ItemQuantityFull")

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CurrencyMonitor:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tSettings =
	{
		nLeft = 70,
		nTop = 130,
		nRight = 680,
		nBottom = 523,
	}
	o.tCharacters = {}
	o.tItems = {}
	
	o.igSessionCash = 0

    return o
end

function CurrencyMonitor:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- CurrencyMonitor OnLoad
-----------------------------------------------------------------------------------------------
function CurrencyMonitor:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CurrencyMonitor.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- CurrencyMonitor OnDocLoaded
-----------------------------------------------------------------------------------------------
function CurrencyMonitor:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CurrencyMonitorForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end

		self.wndMain:SetAnchorOffsets(self.tSettings.nLeft, self.tSettings.nTop, self.tSettings.nRight, self.tSettings.nBottom)

		
		self.wndMaximized = self.wndMain:FindChild("Maximized")
		self.wndMinimized = self.wndMain:FindChild("Minimized")
		
		--lf.wndMaximized:Show(true, true)
		--lf.wndMinimized:Show(false, true)
		
		if bIsMaximized then
			self:OnMaximize()
		else
			self:OnMinimize()
		end
				
		self.wndRealm 				= self.wndMain:FindChild("Maximized:Realm")		
		self.wndCurrenySelection 	= self.wndMain:FindChild("Maximized:ContainerCurrencySelection")
		-- CAN NOT FIND RENOWN/PRESTIGE/ELDER GEM SPRITE
		self.wndCurrenySelection:FindChild("BtnGems:EG"):SetMoneySystem(Money.CodeEnumCurrencyType.ElderGems)
		self.wndCurrenySelection:FindChild("BtnPrestige:P"):SetMoneySystem(Money.CodeEnumCurrencyType.Prestige)
		self.wndCurrenySelection:FindChild("BtnRenown:R"):SetMoneySystem(Money.CodeEnumCurrencyType.Renown)
		--
		self.wndCharacterList 		= self.wndMain:FindChild("Maximized:CharacterList")
		self.wndCashExile 			= self.wndMain:FindChild("Maximized:ContainerCashLabel:ContainerExile")
		self.wndCashDominion 		= self.wndMain:FindChild("Maximized:ContainerCashLabel:ContainerDominion")
		self.wndCashSession 		= self.wndMain:FindChild("Maximized:ContainerCashLabel:ContainerSession")
		
		self.wndCredd				= self.wndMain:FindChild("Maximized:ContainerCREDD:Text")
		self.wndCredd:SetText(AccountItemLib.GetAccountCurrency(AccountItemLib.CodeEnumAccountCurrency.CREDD).."x  C.R.E.D.D available")
	    
		self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)


		-- Do additional Addon initialization here
		
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("ToggleCurrencyMonitor", "OnToggleCurrencyMonitor", self)
	
		Apollo.RegisterTimerHandler("LoadStatus", "OnLoadStatus", self)
		Apollo.CreateTimer("LoadStatus", 0.5, true)

		
	end
end

-----------------------------------------------------------------------------------------------
-- CurrencyMonitor Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

function CurrencyMonitor:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Realm then	   
		return nil
    end
	
	local tSave = {}	
	local dataBuffer = {}
	local settingsBuffer = {}	
	local left, top, right, bottom = self.wndMain:GetAnchorOffsets()
	
	for k,v in pairs(self.tCharacters) do
		dataBuffer[k] =
		{
			level 		= v.level,
			class 		= v.class,
			faction 	= v.faction,
			cash 		= v.cash,
			vouchers 	= v.vouchers,
			gems 		= v.gems,
			renown 		= v.renown,
			prestige 	= v.prestige
		}
	end
	
	settingsBuffer = 
	{
		isMaximized = bIsMaximized,
		nLeft 	= left,--left,
		nTop 	= top,
		nRight 	= right,
		nBottom = bottom,
	}
	--tSave.settings = self.tSettings
	
	tSave.settings 	= settingsBuffer
	tSave.data 		= dataBuffer

	
	return tSave
end

function CurrencyMonitor:OnRestore(eLevel, tData)

	local dataBuffer 		= tData.data
	local settingsBuffer 	= tData.settings	

	for k,v in pairs(dataBuffer) do
		self.tCharacters[k] =
		{
			level = v.level,
			class = v.class,
			faction = v.faction,
			cash = v.cash,
			vouchers = v.vouchers,
			gems = v.gems,
			renown = v.renown,
			prestige = v.prestige
		}
	end
	
	self.tSettings = settingsBuffer	
	bIsMaximized = settingsBuffer.isMaximized
end

--

function CurrencyMonitor:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "CurrencyMonitor", {"ToggleCurrencyMonitor", "", "CRB_Basekit:kitIcon_Holo_HazardTemperature"})
end

function CurrencyMonitor:OnToggleCurrencyMonitor()
	self:PopulateCharacterList(strCurrencySel)
	
	local left, top, right, bottom = self.wndMain:GetAnchorOffsets()
			
	self.tSettings =
	{
		nLeft = left,
		nTop = top,
		nRight = right,
		nBottom = bottom,
	}

	if bIsMaximized then
		self:OnMaximize()
	else
		self:OnMinimize()
	end
		
	self.wndMain:Show(not self.wndMain:IsVisible())
end

-- --------------------------------------------------------------------------------------------

function CurrencyMonitor:OnLoadStatus()
	if GameLib.GetPlayerUnit() then
		self:GatherCurrencyInfo()
	end
end

function CurrencyMonitor:GatherCurrencyInfo()

	local strName = GameLib.GetPlayerUnit():GetName()
	local iCash = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Credits):GetAmount()
	local iGems = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.ElderGems):GetAmount()
	local iVouchers = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.CraftingVouchers):GetAmount()
	local iPrestige = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Prestige):GetAmount()
	local iRenown = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Renown):GetAmount()

	local iLevel = GameLib.GetPlayerUnit():GetLevel()
	local iClassId = GameLib.GetPlayerUnit():GetClassId()
	local strRealm = GameLib.GetRealmName()
	local iFaction = GameLib.GetPlayerUnit():GetFaction()
	
	if bNewSession then
		iSessionCash = iCash
		bNewSession = false
	end
	
	if bSessionReset then
		iSessionCash = iCash
		bSessionReset = false
	end
	
	if bWndMinimized then
	end

	self.tCharacters[strName] =
	{	
		level = iLevel,
		class = iClassId,
		faction = iFaction,
		cash = iCash,
		vouchers = iVouchers,
		gems = iGems,
		renown = iRenown,
		prestige = iPrestige
	}
	
	local nLeft, nTop, nRight, nBottom = self.wndMain:GetAnchorOffsets()
	self.wndMain:SetAnchorOffsets(nLeft, nTop, nRight, nBottom)

	self.wndRealm:SetText("Realm: "..strRealm)

	local sessionCash = iCash - iSessionCash
	self.igSessionCash = sessionCash
	
	if self.igSessionCash >= 0 then
		self.wndCashSession:FindChild("Cash"):SetAmount(self.igSessionCash)
		self.wndCashSession:FindChild("Cash"):SetTextColor(kclrWhite)		
		self.wndMinimized:FindChild("Cash"):SetAmount(self.igSessionCash)
		self.wndMinimized:FindChild("Cash"):SetTextColor(kclrWhite)

	else
		sessionCashBuffer = self.igSessionCash * (-1)
		self.wndCashSession:FindChild("Cash"):SetAmount(sessionCashBuffer)
		self.wndCashSession:FindChild("Cash"):SetTextColor(kclrRed)		
		self.wndMinimized:FindChild("Cash"):SetAmount(sessionCashBuffer)
		self.wndMinimized:FindChild("Cash"):SetTextColor(kclrRed)
	end
	
	
	--Print (GameLib.CodeEnumClass.Warrior)
end

-----------------------------------------------------------------------------------------------
-- CurrencyMonitorForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel/Close button is clicked
function CurrencyMonitor:OnCancel()
	self.wndMain:Close() -- hide the window
end

function CurrencyMonitor:OnMinimize()
	self.wndMaximized:Show(false)
	self.wndMinimized:Show(true)
	bIsMaximized = false
end

function CurrencyMonitor:OnMaximize()
	self.wndMaximized:Show(true)
	self.wndMinimized:Show(false)
	bIsMaximized = true
end

function CurrencyMonitor:OnCurrencySelection(wndHandler, wndControl)

	if wndHandler ~= wndControl then
        return
    end

	local btnName = wndControl:GetName()
	self:PopulateCharacterList(btnName)
end

function CurrencyMonitor:OnSessionCashReset()
	bSessionReset = true
end

function CurrencyMonitor:OnSessionCashMinReset( wndHandler, wndControl, eMouseButton)
	-- 1 = RIGHT MOUSE KEY
	if eMouseButton == 1 then
		bSessionReset = true
	end
end

function CurrencyMonitor:OnCharacterListItemDelete(wndHandler, wndControl)    
	if wndHandler ~= wndControl then
        return
    end

	self.tCharacters[wndControl:FindChild("Name"):GetText()] = nil
	
	bHasExile = false
	bHasDominion = false
	bHasBothFactions = false	
	self:PopulateCharacterList()
end

function CurrencyMonitor:LookupClassIcon(value)
	if value == GameLib.CodeEnumClass.Engineer then
		--Print ("Engineer "..value)
		--return "IconSprites:Icon_Windows_UI_CRB_Engineer"
		return "CRB_Raid:sprRaid_Icon_Class_Engineer"
	elseif value == GameLib.CodeEnumClass.Esper then
		--Print ("Esper "..value)
		--return "IconSprites:Icon_Windows_UI_CRB_Esper"
		return "CRB_Raid:sprRaid_Icon_Class_Esper"
	elseif value == GameLib.CodeEnumClass.Medic then
		--Print ("Medic "..value)
		--return "IconSprites:Icon_Windows_UI_CRB_Medic"
		return "CRB_Raid:sprRaid_Icon_Class_Medic"
	elseif value == GameLib.CodeEnumClass.Spellslinger then
		--Print ("Spellslinger "..value)
		--return "IconSprites:Icon_Windows_UI_CRB_Spellslinger"
		return "CRB_Raid:sprRaid_Icon_Class_Spellslinger"
	elseif value == GameLib.CodeEnumClass.Stalker then
		--Print ("Stalker "..value)
		--return "IconSprites:Icon_Windows_UI_CRB_Stalker"
		return "CRB_Raid:sprRaid_Icon_Class_Stalker"
	elseif value == GameLib.CodeEnumClass.Warrior then
		--Print ("Warrior "..value)
		--return "IconSprites:Icon_Windows_UI_CRB_Warrior"
		return "CRB_Raid:sprRaid_Icon_Class_Warrior"
	end
end

function CurrencyMonitor:PopulateCharacterList(btnSel)
	self:DestroyItemList()
	
	local varBtnSel = btnSel
	
	local idx = 1
	local totalCashExile = 0
	local totalCashDominion = 0
		
	for k,v in pairs(self.tCharacters) do
				
		if v.faction == Unit.CodeEnumFaction.ExilesPlayer then -- 167
			totalCashExile = totalCashExile + v.cash
		else
			totalCashDominion = totalCashDominion + v.cash
		end
		
		local classSprite = self:LookupClassIcon(v.class)		
		
		-- -------------------------------------------------------------------
		
		if v.faction == Unit.CodeEnumFaction.ExilesPlayer then -- 167
			bHasExile = true
		else
			bHasDominion = true
		end
		
		if bHasExile and bHasDominion then
			bHasBothFactions = true
		end

		if not bHasBothFactions then
			if bHasExile then		
				self.wndCashExile:SetAnchorOffsets(0,0,260,44)			
				self.wndCashDominion:Show(false, true)
				self.wndCashExile:Show(true, true)
			else
				self.wndCashDominion:SetAnchorOffsets(0,0,260,44)			
				self.wndCashDominion:Show(true, true)
				self.wndCashExile:Show(false, true)
			end		
			self.wndCashSession:SetAnchorOffsets(0,50,260,97)	
		else 
			self.wndCashExile:SetAnchorOffsets(0,0,260,44)
			self.wndCashDominion:SetAnchorOffsets(0,50,260,94)			
			self.wndCashSession:SetAnchorOffsets(0,100,260,147)
			self.wndCashDominion:Show(true, true)
			self.wndCashExile:Show(true, true)
		end
			
		-- -------------------------------------------------------------------		
		self:AddItem(idx, k, v, classSprite, varBtnSel)
		idx = idx + 1
	end
	
	self.wndCashExile:FindChild("Cash"):SetAmount(totalCashExile)
	self.wndCashDominion:FindChild("Cash"):SetAmount(totalCashDominion)
	self.wndCharacterList:ArrangeChildrenVert()
end

function CurrencyMonitor:AddItem(idx, name, v, sprite, btnSel)

	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndCharacterList, self)
	
	self.tItems[idx] = wnd
	
	local wndLevel = wnd:FindChild("CName:Level")
	local wndName = wnd:FindChild("CName:Name")
	local wndCash = wnd:FindChild("CCash")
	local wndIcon = wnd:FindChild("ClassIcon")
	local wndFaction = wnd:FindChild("FactionIcon")
	
	if wndName and wndCash and wndIcon then
		wndName:SetText(name)
		if btnSel == "BtnCash" or btnSel == "default" then
			wndCash:SetMoneySystem(Money.CodeEnumCurrencyType.Credits)
			wndCash:SetAmount(v.cash)
		elseif btnSel == "BtnGems" then
			wndCash:SetMoneySystem(Money.CodeEnumCurrencyType.ElderGems)
			wndCash:SetAmount(v.gems)
		elseif btnSel == "BtnPrestige" then
			wndCash:SetMoneySystem(Money.CodeEnumCurrencyType.Prestige)
			wndCash:SetAmount(v.prestige)
		elseif btnSel == "BtnVouchers" then
			wndCash:SetMoneySystem(Money.CodeEnumCurrencyType.CraftingVouchers)
			wndCash:SetAmount(v.vouchers)
		elseif btnSel == "BtnRenown" then
			wndCash:SetMoneySystem(Money.CodeEnumCurrencyType.Renown)
			wndCash:SetAmount(v.renown)
		end
		wndIcon:SetSprite(sprite)

		if v.faction == Unit.CodeEnumFaction.ExilesPlayer then
			wndFaction:SetSprite("charactercreate:sprCharC_Ico_Exile")
		else
			wndFaction:SetSprite("charactercreate:sprCharC_Ico_Dominion")
		end
		
		wndLevel:SetText("["..v.level.."]")
	end	
end

function CurrencyMonitor:DestroyItemList()

	for idx,wnd in pairs(self.tItems) do
		wnd:Destroy()
	end
	
	self.tItems = {}
end

-----------------------------------------------------------------------------------------------
-- CurrencyMonitor Instance
-----------------------------------------------------------------------------------------------
local CurrencyMonitorInst = CurrencyMonitor:new()
CurrencyMonitorInst:Init()
